@@ -152,7 +152,6 @@ gs_grab_get_keyboard (GSGrab    *grab,
 
         gs_debug ("Grabbing keyboard widget=%X", (guint32) GDK_WINDOW_XID (window));
 
-#if GTK_CHECK_VERSION(3, 0, 0)
         GList *list, *link;
         GdkDisplay *display = gdk_window_get_display (window);
         GdkDeviceManager *device_manager = gdk_display_get_device_manager (display);
@@ -174,9 +173,6 @@ gs_grab_get_keyboard (GSGrab    *grab,
                                           GDK_CURRENT_TIME);
         }
         g_list_free(list);
-#else
-        status = gdk_keyboard_grab (window, FALSE, GDK_CURRENT_TIME);
-#endif
 
         if (status == GDK_GRAB_SUCCESS) {
                 if (grab->priv->keyboard_grab_window != NULL) {
@@ -211,7 +207,6 @@ gs_grab_get_mouse (GSGrab    *grab,
         cursor = gdk_cursor_new (GDK_BLANK_CURSOR);
 
         gs_debug ("Grabbing mouse widget=%X", (guint32) GDK_WINDOW_XID (window));
-#if GTK_CHECK_VERSION(3, 0, 0)
         GList *list, *link;
         GdkDisplay *display = gdk_window_get_display (window);
         GdkDeviceManager *device_manager = gdk_display_get_device_manager (display);
@@ -229,15 +224,10 @@ gs_grab_get_mouse (GSGrab    *grab,
                                           FALSE,
                                           GDK_KEY_PRESS_MASK |
                                           GDK_KEY_RELEASE_MASK,
-                                          NULL,
+                                          (hide_cursor ? cursor : NULL),
                                           GDK_CURRENT_TIME);
         }
         g_list_free(list);
-#else
-        status = gdk_pointer_grab (window, TRUE, 0, NULL,
-                                   (hide_cursor ? cursor : NULL),
-                                   GDK_CURRENT_TIME);
-#endif
 
         if (status == GDK_GRAB_SUCCESS) {
                 if (grab->priv->mouse_grab_window != NULL) {
@@ -253,11 +243,7 @@ gs_grab_get_mouse (GSGrab    *grab,
                 grab->priv->mouse_hide_cursor = hide_cursor;
         }
 
-#if GTK_CHECK_VERSION(3, 0, 0)
         g_object_unref (cursor);
-#else
-        gdk_cursor_unref (cursor);
-#endif
 
         return status;
 }
@@ -278,7 +264,6 @@ gs_grab_release_keyboard (GSGrab *grab)
 {
         gs_debug ("Ungrabbing keyboard");
 
-#if GTK_CHECK_VERSION(3, 0, 0)
         GList *list, *link;
         GdkDisplay *display = gdk_display_get_default ();
         GdkDeviceManager *device_manager = gdk_display_get_device_manager (display);
@@ -293,9 +278,6 @@ gs_grab_release_keyboard (GSGrab *grab)
                 gdk_device_ungrab(device, GDK_CURRENT_TIME);
         }
         g_list_free(list);
-#else
-        gdk_keyboard_ungrab (GDK_CURRENT_TIME);
-#endif
 
         gs_grab_keyboard_reset (grab);
 
@@ -319,7 +301,6 @@ gs_grab_release_mouse (GSGrab *grab)
 {
         gs_debug ("Ungrabbing pointer");
 
-#if GTK_CHECK_VERSION(3, 0, 0)
         GList *list, *link;
         GdkDisplay *display = gdk_display_get_default ();
         GdkDeviceManager *device_manager = gdk_display_get_device_manager (display);
@@ -334,9 +315,6 @@ gs_grab_release_mouse (GSGrab *grab)
                 gdk_device_ungrab(device, GDK_CURRENT_TIME);
         }
         g_list_free(list);
-#else
-        gdk_pointer_ungrab (GDK_CURRENT_TIME);
-#endif
 
         gs_grab_mouse_reset (grab);
 
@@ -356,7 +334,6 @@ gs_grab_move_mouse (GSGrab    *grab,
 
         /* if the pointer is not grabbed and we have a
            mouse_grab_window defined then we lost the grab */
-#if GTK_CHECK_VERSION(3, 0, 0)
         GList *list, *link;
         GdkDisplay *display = gdk_display_get_default ();
         GdkDeviceManager *device_manager = gdk_display_get_device_manager (display);
@@ -372,11 +349,6 @@ gs_grab_move_mouse (GSGrab    *grab,
                 }
         }
         g_list_free(list);
-#else
-        if (! gdk_pointer_is_grabbed ()) {
-                gs_grab_mouse_reset (grab);
-        }
-#endif
 
         if (grab->priv->mouse_grab_window == window) {
                 gs_debug ("Window %X is already grabbed, skipping",
@@ -497,11 +469,7 @@ gs_grab_nuke_focus (void)
 
         XSetInputFocus (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), None, RevertToNone, CurrentTime);
 
-#if GTK_CHECK_VERSION(3, 0, 0)
         gdk_error_trap_pop_ignored ();
-#else
-        gdk_error_trap_pop ();
-#endif
 }
 
 void
@@ -647,14 +615,10 @@ gs_grab_grab_root (GSGrab  *grab,
         gs_debug ("Grabbing the root window");
 
         display = gdk_display_get_default ();
-#if GTK_CHECK_VERSION(3, 0, 0)
         GdkDeviceManager *device_manager = gdk_display_get_device_manager (display);
         GdkDevice *pointer = gdk_device_manager_get_client_pointer (device_manager);
         gint x = -1, y = -1;
         gdk_device_get_position (pointer, &screen, &x, &y);
-#else
-        gdk_display_get_pointer (display, &screen, NULL, NULL, NULL);
-#endif
         root = gdk_screen_get_root_window (screen);
 
         res = gs_grab_grab_window (grab, root, screen, hide_cursor);
