@@ -42,13 +42,18 @@ struct GSManagerPrivate
 {
         GSList      *windows;
 
+        /* Configuration */
+        guint        lock_after;
+
         /* State */
         guint        active : 1;
         guint        visible : 1;
-
-        time_t       activate_time;
+        guint        blank : 1;
+        guint        closed : 1;
+        guint        show_content : 1;
 
         guint        greeter_timeout_id;
+        guint        lock_timeout_id;
 
         GSGrab      *grab;
 };
@@ -56,6 +61,7 @@ struct GSManagerPrivate
 enum {
         ACTIVATED,
         SWITCH_GREETER,
+        LOCK,
         LAST_SIGNAL
 };
 
@@ -135,6 +141,17 @@ gs_manager_class_init (GSManagerClass *klass)
                               G_TYPE_NONE,
                               0);
 
+        signals [LOCK] =
+                g_signal_new ("lock",
+                              G_TYPE_FROM_CLASS (object_class),
+                              G_SIGNAL_RUN_LAST,
+                              G_STRUCT_OFFSET (GSManagerClass, lock),
+                              NULL,
+                              NULL,
+                              g_cclosure_marshal_VOID__VOID,
+                              G_TYPE_NONE,
+                              0);
+
         g_object_class_install_property (object_class,
                                          PROP_ACTIVE,
                                          g_param_spec_boolean ("active",
@@ -153,8 +170,10 @@ gs_manager_init (GSManager *manager)
 
         manager->priv->grab = gs_grab_new ();
 
-	/* Assume we are the visible session on start. */
-	manager->priv->visible = TRUE;
+        /* Assume we are the visible session on start. */
+        manager->priv->visible = TRUE;
+
+        manager->priv->lock_after = 5;
 }
 
 
@@ -169,13 +188,9 @@ manager_maybe_grab_window (GSManager *manager,
         gboolean    grabbed;
 
         display = gdk_display_get_default ();
-#if GTK_CHECK_VERSION(3, 0, 0)
         GdkDeviceManager *device_manager = gdk_display_get_device_manager (display);
         GdkDevice *pointer = gdk_device_manager_get_client_pointer (device_manager);
         gdk_device_get_position (pointer, &screen, &x, &y);
-#else
-        gdk_display_get_pointer (display, &screen, &x, &y, NULL);
-#endif
         monitor = gdk_screen_get_monitor_at_point (screen, x, y);
 
         gdk_flush ();
@@ -186,7 +201,7 @@ manager_maybe_grab_window (GSManager *manager,
                 gs_grab_move_to_window (manager->priv->grab,
                                         gs_window_get_gdk_window (window),
                                         gs_window_get_screen (window),
-                                        FALSE);
+                                        TRUE);
                 grabbed = TRUE;
         }
 
@@ -213,77 +228,35 @@ window_map_event_cb (GSWindow  *window,
 {
         gs_debug ("Handling window map_event event");
 
+        /* FIXME: only emit signal once */
+        g_signal_emit (manager, signals [ACTIVATED], 0);
+
         manager_maybe_grab_window (manager, window);
 
         return FALSE;
 }
 
 static void
-window_map_cb (GSWindow  *window,
-               GSManager *manager)
-{
-        gs_debug ("Handling window map event");
-}
-
-static void
-window_unmap_cb (GSWindow  *window,
+content_draw_cb (GtkWidget *widget,
+                 cairo_t   *cr,
                  GSManager *manager)
 {
-        gs_debug ("window unmapped!");
+        cairo_set_source_rgba (cr, 0.0, 0.0, 0.0, 1.0);
+        cairo_paint (cr);
+
+        if (manager->priv->show_content)
+                content_draw (widget, cr);
 }
 
-static void
-manager_show_window (GSManager *manager,
-                     GSWindow  *window)
-{
-        manager->priv->activate_time = time (NULL);
-
-        /* FIXME: only emit signal once */
-        g_signal_emit (manager, signals [ACTIVATED], 0);
-}
-
-static void
-window_show_cb (GSWindow  *window,
-                GSManager *manager)
-{
-
-        g_return_if_fail (manager != NULL);
-        g_return_if_fail (GS_IS_MANAGER (manager));
-        g_return_if_fail (window != NULL);
-        g_return_if_fail (GS_IS_WINDOW (window));
-
-        gs_debug ("Handling window show");
-        manager_show_window (manager, window);
-}
-
-#if !GTK_CHECK_VERSION(3, 0, 0)
-static void
-content_expose_cb (GtkWidget *widget,
-                  GdkEvent  *event,
-                  gpointer   user_data)
-{
-        cairo_t *cr = gdk_cairo_create (gtk_widget_get_window (widget));
-        content_draw_cb (widget, cr, user_data);
-        cairo_destroy (cr);
-}
-#endif
-
 static void
 disconnect_window_signals (GSManager *manager,
                            GSWindow  *window)
 {
         GtkWidget *drawing_area = gs_window_get_drawing_area (window);
 
-#if GTK_CHECK_VERSION(3, 0, 0)
-        g_signal_handlers_disconnect_by_func (drawing_area, content_draw_cb, NULL);
-#else
-        g_signal_handlers_disconnect_by_func (drawing_area, content_expose_cb, NULL);
-#endif
+        g_signal_handlers_disconnect_by_func (drawing_area, content_draw_cb, manager);
 
-        g_signal_handlers_disconnect_by_func (window, window_show_cb, manager);
-        g_signal_handlers_disconnect_by_func (window, window_map_cb, manager);
         g_signal_handlers_disconnect_by_func (window, window_map_event_cb, manager);
-        g_signal_handlers_disconnect_by_func (window, window_unmap_cb, manager);
         g_signal_handlers_disconnect_by_func (window, window_grab_broken_cb, manager);
 }
 
@@ -301,24 +274,13 @@ connect_window_signals (GSManager *manager,
 {
         GtkWidget *drawing_area = gs_window_get_drawing_area (window);
 
-#if GTK_CHECK_VERSION(3, 0, 0)
         g_signal_connect_object (drawing_area, "draw",
-                                 G_CALLBACK (content_draw_cb), NULL, 0);
-#else
-        g_signal_connect_object (drawing_area, "expose-event",
-                                 G_CALLBACK (content_expose_cb), NULL, 0);
-#endif
+                                 G_CALLBACK (content_draw_cb), manager, 0);
 
         g_signal_connect_object (window, "destroy",
                                  G_CALLBACK (window_destroyed_cb), manager, 0);
-        g_signal_connect_object (window, "show",
-                                 G_CALLBACK (window_show_cb), manager, G_CONNECT_AFTER);
-        g_signal_connect_object (window, "map",
-                                 G_CALLBACK (window_map_cb), manager, G_CONNECT_AFTER);
         g_signal_connect_object (window, "map_event",
                                  G_CALLBACK (window_map_event_cb), manager, G_CONNECT_AFTER);
-        g_signal_connect_object (window, "unmap",
-                                 G_CALLBACK (window_unmap_cb), manager, G_CONNECT_AFTER);
         g_signal_connect_object (window, "grab_broken_event",
                                  G_CALLBACK (window_grab_broken_cb), manager, G_CONNECT_AFTER);
 }
@@ -392,6 +354,15 @@ on_screen_monitors_changed (GdkScreen *screen,
                 gdk_flush ();
                 gdk_x11_ungrab_server ();
         }
+
+	for (l = manager->priv->windows; l != NULL; l = l->next) {
+		GdkScreen *this_screen;
+
+		this_screen = gs_window_get_screen (GS_WINDOW (l->data));
+		if (this_screen == screen) {
+			gtk_widget_queue_resize (GTK_WIDGET (l->data));
+		}
+	}
 }
 
 static void
@@ -426,6 +397,87 @@ gs_manager_destroy_windows (GSManager *manager)
         manager->priv->windows = NULL;
 }
 
+static gboolean
+switch_greeter_timeout (GSManager *manager)
+{
+        manager->priv->greeter_timeout_id = 0;
+
+        gs_debug ("Switch to greeter timeout");
+
+        g_signal_emit (manager, signals [SWITCH_GREETER], 0);
+
+        return FALSE;
+}
+
+static void
+gs_manager_timed_switch (GSManager *manager)
+{
+        if (manager->priv->greeter_timeout_id != 0) {
+                gs_debug ("Trying to start an active switch to greeter timer");
+                return;
+        }
+
+        gs_debug ("Start switch to greeter timer");
+
+        manager->priv->greeter_timeout_id = g_timeout_add_seconds (10,
+                                                                   (GSourceFunc)switch_greeter_timeout,
+                                                                   manager);
+}
+
+static void
+gs_manager_stop_switch (GSManager *manager)
+{
+        if (manager->priv->greeter_timeout_id != 0) {
+                gs_debug ("Stop switch to greeter timer");
+
+                g_source_remove (manager->priv->greeter_timeout_id);
+                manager->priv->greeter_timeout_id = 0;
+        }
+}
+
+static gboolean
+lock_timeout (GSManager *manager)
+{
+        manager->priv->lock_timeout_id = 0;
+
+        gs_debug ("Lock timeout");
+
+        g_signal_emit (manager, signals [LOCK], 0);
+
+        return FALSE;
+}
+
+static void
+gs_manager_timed_lock (GSManager *manager)
+{
+        if (manager->priv->lock_after == 0) {
+                gs_debug ("Lock after disabled");
+                return;
+        }
+
+        if (manager->priv->lock_timeout_id != 0) {
+                gs_debug ("Trying to start an active lock timer");
+                return;
+        }
+
+        gs_debug ("Start lock timer");
+
+        manager->priv->lock_timeout_id = g_timeout_add_seconds (manager->priv->lock_after,
+                                                                (GSourceFunc)lock_timeout,
+                                                                manager);
+}
+
+static void
+gs_manager_stop_lock (GSManager *manager)
+{
+        if (manager->priv->lock_timeout_id != 0) {
+                gs_debug ("Stop lock timer");
+
+                g_source_remove (manager->priv->lock_timeout_id);
+                manager->priv->lock_timeout_id = 0;
+        }
+}
+
 static void
 gs_manager_finalize (GObject *object)
 {
@@ -443,7 +495,9 @@ gs_manager_finalize (GObject *object)
         gs_manager_destroy_windows (manager);
 
         manager->priv->active = FALSE;
-        manager->priv->activate_time = 0;
+
+        gs_manager_stop_switch (manager);
+        gs_manager_stop_lock (manager);
 
         g_object_unref (manager->priv->grab);
 
@@ -521,44 +575,6 @@ show_windows (GSList *windows)
         }
 }
 
-static gboolean
-switch_greeter_timeout (GSManager *manager)
-{
-        manager->priv->greeter_timeout_id = 0;
-
-        gs_debug ("Switch to greeter timeout");
-
-        g_signal_emit (manager, signals [SWITCH_GREETER], 0);
-
-        return FALSE;
-}
-
-static void
-gs_manager_timed_switch (GSManager *manager)
-{
-        if (manager->priv->greeter_timeout_id != 0) {
-                gs_debug ("Trying to start an active switch to greeter timer");
-                return;
-        }
-
-        gs_debug ("Start switch to greeter timer");
-
-        manager->priv->greeter_timeout_id = g_timeout_add_seconds (10,
-                                                                   (GSourceFunc)switch_greeter_timeout,
-                                                                   manager);
-}
-
-static void
-gs_manager_stop_switch (GSManager *manager)
-{
-        if (manager->priv->greeter_timeout_id != 0) {
-                gs_debug ("Stop switch to greeter timer");
-
-                g_source_remove (manager->priv->greeter_timeout_id);
-                manager->priv->greeter_timeout_id = 0;
-        }
-}
-
 static gboolean
 gs_manager_activate (GSManager *manager)
 {
@@ -585,10 +601,12 @@ gs_manager_activate (GSManager *manager)
 
         show_windows (manager->priv->windows);
 
-        if (manager->priv->visible) {
+        if (manager->priv->visible && !manager->priv->blank && !manager->priv->closed) {
                 gs_manager_timed_switch (manager);
         }
 
+        gs_manager_stop_lock (manager);
+
         return TRUE;
 }
 
@@ -609,9 +627,13 @@ gs_manager_deactivate (GSManager *manager)
 
         gs_manager_stop_switch (manager);
 
+        if (manager->priv->blank) {
+                gs_manager_timed_lock (manager);
+        }
+
         /* reset state */
         manager->priv->active = FALSE;
-        manager->priv->activate_time = 0;
+        manager->priv->show_content = FALSE;
 
         return TRUE;
 }
@@ -641,12 +663,12 @@ gs_manager_get_active (GSManager *manager)
 }
 
 void
-gs_manager_set_session_visible (GSManager  *manager,
-                                gboolean    visible)
+gs_manager_set_session_visible (GSManager *manager,
+                                gboolean   visible)
 {
         manager->priv->visible = visible;
 
-        if (manager->priv->active && visible) {
+        if (manager->priv->active && visible && !manager->priv->blank && !manager->priv->closed) {
                 gs_manager_timed_switch (manager);
         } else {
                 gs_manager_stop_switch (manager);
@@ -661,3 +683,63 @@ gs_manager_get_session_visible (GSManager *manager)
 
         return manager->priv->visible;
 }
+
+void
+gs_manager_set_blank_screen (GSManager *manager,
+                             gboolean   blank)
+{
+        manager->priv->blank = blank;
+
+        if (!manager->priv->active && blank) {
+                gs_manager_timed_lock (manager);
+        } else {
+                gs_manager_stop_lock (manager);
+                if (manager->priv->active && manager->priv->visible && !manager->priv->closed) {
+                        gs_manager_timed_switch (manager);
+                }
+        }
+}
+
+gboolean
+gs_manager_get_blank_screen (GSManager *manager)
+{
+        g_return_val_if_fail (manager != NULL, FALSE);
+        g_return_val_if_fail (GS_IS_MANAGER (manager), FALSE);
+
+        return manager->priv->blank;
+}
+
+void
+gs_manager_set_lid_closed (GSManager *manager,
+                           gboolean   closed)
+{
+        manager->priv->closed = closed;
+
+        if (manager->priv->active && manager->priv->visible && !manager->priv->blank && !closed) {
+                gs_manager_timed_switch (manager);
+        } else {
+                gs_manager_stop_switch (manager);
+        }
+}
+
+void
+gs_manager_set_lock_after (GSManager *manager,
+                           guint      lock_after)
+{
+        manager->priv->lock_after = lock_after;
+}
+
+void
+gs_manager_show_content (GSManager *manager)
+{
+        GSList *l;
+
+        if (manager->priv->show_content)
+                return;
+
+        manager->priv->show_content = TRUE;
+
+        for (l = manager->priv->windows; l; l = l->next) {
+                gtk_widget_queue_draw (gs_window_get_drawing_area (l->data));
+        }
+}
